@@ -1,21 +1,23 @@
 package TestApp;
 
 use strict;
+use warnings;
 use Catalyst qw/
     Session
     Session::Store::File 
     Session::State::Cookie
     
-    +CatalystX::I18N::Role::Base
-    +CatalystX::I18N::Role::DateTime
-    +CatalystX::I18N::Role::Maketext
-    +CatalystX::I18N::Role::GetLocale
-    +CatalystX::I18N::Role::NumberFormat
+    +CatalystX::I18N::Role::All
 /;
-use CatalystX::RoleApplicator;
+#    +CatalystX::I18N::Role::Base
+#    +CatalystX::I18N::Role::DateTime
+#    +CatalystX::I18N::Role::Maketext
+#    +CatalystX::I18N::Role::GetLocale
+#    +CatalystX::I18N::Role::NumberFormat
+#use CatalystX::RoleApplicator;
 
-__PACKAGE__->apply_request_class_roles(qw/CatalystX::I18N::TraitFor::Request/);
-__PACKAGE__->apply_response_class_roles(qw/CatalystX::I18N::TraitFor::Response/);
+#__PACKAGE__->apply_request_class_roles(qw/CatalystX::I18N::TraitFor::Request/);
+#__PACKAGE__->apply_response_class_roles(qw/CatalystX::I18N::TraitFor::Response/);
 
 our $VERSION = '0.01';
 
@@ -42,6 +44,7 @@ TestApp->config(
                 timezone                => 'Europe/Vienna',
                 inherits                => 'de',
                 format_datetime         => 'dd.MM.yyyy uma HH:mm',
+                mon_decimal_point       => ',',
             },
             'de_DE'                 => {
                 inherits                => 'de',
